// Copyright 2020, 2023 Global Virtual Airlines Group. All Rights Reserved.
package org.deltava.dao;

import java.sql.*;
import java.util.*;

import org.deltava.beans.econ.*;

import org.deltava.util.cache.*;

/**
 * A Data Access Object to read Elite program definitions. 
 * @author Luke
 * @version 11.1
 * @since 9.2
 */

abstract class EliteDAO extends DAO {
	
	/**
	 * The EliteLevel cache.
	 */
	protected static final Cache<EliteLevel> _lvlCache = CacheManager.get(EliteLevel.class, "EliteLevel");
	
	/**
	 * The EliteStatus cache.
	 */
	protected static final Cache<EliteStatus> _stCache = CacheManager.get(EliteStatus.class, "EliteStatus");

	/**
	 * Initializes the Data Access Object.
	 * @param c the JDBC connection to use
	 */
	protected EliteDAO(Connection c) {
		super(c);
	}
	
	/**
	 * Returns an Elite status level for a particular year.
	 * @param name the level name
	 * @param year the year
	 * @param dbName the database name
	 * @return an EliteLevel, or null if not found
	 * @throws DAOException if a JDBC error occurs
	 */
	public EliteLevel get(String name, int year, String dbName) throws DAOException {
		
		// Check the cache
		EliteLevel lvl = _lvlCache.get(new EliteLevel(year, name, dbName).cacheKey());
		if (lvl != null)
			return lvl;
		
		// Build the SQL statement
		String db = formatDBName(dbName);
		StringBuilder sqlBuf = new StringBuilder("SELECT EL.*, ? FROM ");
		sqlBuf.append(db);
		sqlBuf.append(".ELITE_LEVELS EL WHERE (NAME=?) AND (YR=?) LIMIT 1");
		
		try (PreparedStatement ps = prepareWithoutLimits(sqlBuf.toString())) {
			ps.setString(1, db);
			ps.setString(2, name);
			ps.setInt(3, year);
			return executeLevel(ps).stream().findFirst().orElse(null);
		} catch (SQLException se) {
			throw new DAOException(se);
		}
	}
	
	/**
	 * Helper method to parse elite level result sets.
	 * @param ps the PreparedStatement to execute
	 * @return a List of EliteLevel beans
	 * @throws SQLException if a JDBC error occurs
	 */
	protected static List<EliteLevel> executeLevel(PreparedStatement ps) throws SQLException {
		List<EliteLevel> results = new ArrayList<EliteLevel>();
		try (ResultSet rs = ps.executeQuery()) {
			while (rs.next()) {
				EliteLevel lvl = new EliteLevel(rs.getInt(2), rs.getString(1), rs.getString(11));
				lvl.setStatisticsStartDate(expandDate(rs.getDate(3)));
				lvl.setLegs(rs.getInt(4));
				lvl.setDistance(rs.getInt(5));
				lvl.setPoints(rs.getInt(6));
				lvl.setBonusFactor(rs.getInt(7) / 100.0f);
				lvl.setColor(rs.getInt(8));
				lvl.setTargetPercentile(rs.getInt(9));
				lvl.setVisible(rs.getBoolean(10));
				results.add(lvl);
				_lvlCache.add(lvl);
			}
		}
		
		return results;
	}

	/**
	 * Helper method to populate EliteLevel beans.
	 * @param data a Collection of EliteLeveBeans
	 * @throws DAOException if a JDBC error occurs
	 */
	protected void populateLevels(Collection<? extends EliteLevelBean> data) throws DAOException {
		for (EliteLevelBean eb : data) {
			EliteLevel lvl = get(eb.getLevel().getName(), eb.getLevel().getYear(), eb.getLevel().getOwner());
			if (lvl != null)
				eb.setLevel(lvl);
		}
	}
}